==> dga_rating-1.0.0.zip: dga_rating/js/rating.js <==
(function (Drupal, once) {
  "use strict";

  // DGA Rating Widget Script

  function pctForRating(r) {
    return (Math.max(0, Math.min(5, Number(r) || 0)) / 5) * 100;
  }

  Drupal.behaviors.ratingWidget = {
    attach: function(context, settings) {
      // Find widgets - use direct query if once() fails
      let widgets = [];
      try {
        widgets = once("ratingWidget", "[data-rating-widget]", context);
      } catch (e) {
        // Fallback to direct query
      }

      if (widgets.length === 0) {
        const direct = (context.querySelectorAll || document.querySelectorAll).call(context === document ? document : context, "[data-rating-widget]");
        widgets = Array.from(direct || []);
      }

      if (widgets.length === 0) {
        return;
      }

      widgets.forEach((root) => {
        if (root.hasAttribute('data-rw-initialized')) {
          return;
        }
        root.setAttribute('data-rw-initialized', 'true');

        const rowClosed = root.querySelector("[data-rw-closed]");
        const rowOpen = root.querySelector("[data-rw-opened]");
        const rowSubmitted = root.querySelector("[data-rw-submitted]");
        const btnOpen = root.querySelector("[data-rw-open]");
        const btnClose = root.querySelector("[data-rw-close]");
        const btnSubmit = root.querySelector("[data-rw-submit]");
        const avgText = root.querySelector("[data-rw-average-text]");
        const avgFill = root.querySelector("[data-rw-average-fill]");
        const reviewsEl = root.querySelector("[data-rw-reviews]");
        const starWrap = root.querySelector("[data-rw-stars]");
        const starBtns = root.querySelectorAll("[data-rw-star]");
        const feedbackEl = root.querySelector("[data-rw-feedback]");
        const yourRatingText = root.querySelector("[data-rw-your-rating]");
        const yourFill = root.querySelector("[data-rw-your-fill]");

        let average = Number(root.getAttribute("data-average")) || 0;
        let reviews = root.getAttribute("data-reviews") || "0";
        let url = root.getAttribute("data-url") || window.location.pathname;
        url = url.trim().replace(/\/+$/, '') || '/';
        if (url.match(/^\/[a-z]{2}\//)) {
          url = url.replace(/^\/[a-z]{2}/, '');
          url = url || '/';
        }

        const entityType = root.getAttribute("data-entity-type") || "";
        const entityId = root.getAttribute("data-entity-id") || "";

        const state = { rating: 0, feedback: "", open: false, submitted: false };

        function setClosedView() {
          if (rowClosed) rowClosed.hidden = false;
          if (rowOpen) rowOpen.hidden = true;
          if (rowSubmitted) rowSubmitted.hidden = true;
        }
        function setOpenView() {
          if (rowClosed) rowClosed.hidden = true;
          if (rowOpen) rowOpen.hidden = false;
          if (rowSubmitted) rowSubmitted.hidden = true;
        }
        function setSubmittedView() {
          if (rowClosed) rowClosed.hidden = true;
          if (rowOpen) rowOpen.hidden = true;
          if (rowSubmitted) rowSubmitted.hidden = false;
        }
        function paintAverage() {
          if (avgText) avgText.textContent = average.toFixed(1);
          if (avgFill) avgFill.style.width = pctForRating(average) + "%";
          if (reviewsEl) reviewsEl.textContent = reviews + (reviews === "1" ? " review" : " reviews");
        }
        function paintYourRating() {
          if (yourRatingText) yourRatingText.textContent = String(state.rating);
          if (yourFill) yourFill.style.width = pctForRating(state.rating) + "%";
          starBtns.forEach((btn) => {
            const v = Number(btn.getAttribute("data-value"));
            btn.setAttribute("aria-checked", v === state.rating ? "true" : "false");
            btn.classList.toggle("is-active", v <= state.rating);
          });
        }

        // Events
        if (btnOpen) {
          btnOpen.addEventListener("click", (e) => {
            e.preventDefault();
            state.open = true;
            setOpenView();
          });
        }

        if (btnClose) {
          btnClose.addEventListener("click", () => {
            state.open = false;
            setClosedView();
          });
        }

        if (feedbackEl) {
          feedbackEl.addEventListener("input", (e) => {
            state.feedback = e.target.value;
          });
        }

        starBtns.forEach((btn) => {
          btn.addEventListener("click", () => {
            state.rating = Number(btn.getAttribute("data-value"));
            paintYourRating();
          });
        });

        // The global handler below will attach to the submit button
        // No need for widget-specific handler since global one works reliably
      });
    },
  };

  // Fallback for delayed content
  setTimeout(() => {
    if (typeof Drupal !== 'undefined' && Drupal.behaviors && Drupal.behaviors.ratingWidget) {
      Drupal.behaviors.ratingWidget.attach(document, Drupal.settings || {});
    }
  }, 1000);
  
  // GLOBAL HANDLER - Direct button handler that works independently
  function attachGlobalSubmitHandler() {
    const submitButtons = document.querySelectorAll('[data-rw-submit]');
    
    submitButtons.forEach(function(btn) {
      if (btn.hasAttribute('data-global-handler-attached')) {
        return;
      }
      
      btn.setAttribute('data-global-handler-attached', 'true');
      
      btn.onclick = function(e) {
        e.preventDefault();
        e.stopPropagation();
        e.stopImmediatePropagation();
        
        // Find widget root
        const widget = btn.closest('[data-rating-widget]');
        if (!widget) {
          console.error('DGA Rating: Widget not found');
          alert('ERROR: Widget not found');
          return false;
        }
        
        // Get rating from stars
        let rating = 0;
        const activeStars = widget.querySelectorAll('[data-rw-star].is-active');
        if (activeStars.length > 0) {
          rating = activeStars.length;
        } else {
          const checkedStar = widget.querySelector('[data-rw-star][aria-checked="true"]');
          if (checkedStar) {
            rating = Number(checkedStar.getAttribute('data-value')) || 0;
          }
        }
        
        if (rating < 1 || rating > 5) {
          alert("Please select a rating before submitting.");
          return false;
        }
        
        // Get feedback
        const feedbackEl = widget.querySelector('[data-rw-feedback]');
        const feedback = feedbackEl ? feedbackEl.value : '';
        
        // Get URL
        let url = widget.getAttribute('data-url') || window.location.pathname;
        url = url.trim().replace(/\/+$/, '') || '/';
        if (url.match(/^\/[a-z]{2}\//)) {
          url = url.replace(/^\/[a-z]{2}/, '');
          url = url || '/';
        }
        
        // Disable button
        const originalText = btn.textContent;
        btn.disabled = true;
        btn.textContent = "Submitting...";
        
        // Prepare payload
        const payload = {
          rating: rating,
          feedback: feedback || '',
          url: url || '/',
          entity_type: 'node',
          entity_id: 0
        };
        
        // Send request
        const xhr = new XMLHttpRequest();
        xhr.open('POST', '/dga-rating/submit', false);
        xhr.setRequestHeader('Content-Type', 'application/json');
        
        try {
          xhr.send(JSON.stringify(payload));
          
          let data;
          try {
            data = JSON.parse(xhr.responseText);
          } catch (e) {
            console.error('DGA Rating: JSON Parse Error:', e);
            alert('ERROR: Invalid JSON response: ' + xhr.responseText.substring(0, 200));
            btn.disabled = false;
            btn.textContent = originalText;
            return false;
          }
          
          // Validate response
          if (xhr.status === 200 && data.success && data.rating_id) {
            
            // Update widget UI
            const rowClosed = widget.querySelector('[data-rw-closed]');
            const rowOpen = widget.querySelector('[data-rw-opened]');
            const rowSubmitted = widget.querySelector('[data-rw-submitted]');
            
            if (rowClosed) rowClosed.hidden = true;
            if (rowOpen) rowOpen.hidden = true;
            if (rowSubmitted) {
              rowSubmitted.hidden = false;
              const yourRating = rowSubmitted.querySelector('[data-rw-your-rating]');
              if (yourRating)               yourRating.textContent = rating;
            }
          } else {
            console.error('DGA Rating: Submission failed - Status:', xhr.status, 'Success:', data.success, 'Rating ID:', data.rating_id);
            alert('ERROR: Status ' + xhr.status + '\nMessage: ' + (data.message || 'Unknown error'));
            btn.disabled = false;
            btn.textContent = originalText;
          }
        } catch (err) {
          console.error('DGA Rating: Exception during submission:', err);
          alert('EXCEPTION: ' + err.message + '\nStatus: ' + xhr.status);
          btn.disabled = false;
          btn.textContent = originalText;
        }
        
        return false;
      };
    });
  }
  
  // Attach on DOM ready and once after delay
  if (document.readyState === 'loading') {
    document.addEventListener('DOMContentLoaded', attachGlobalSubmitHandler);
  } else {
    attachGlobalSubmitHandler();
  }
  
  // Single delayed attach as fallback
  setTimeout(attachGlobalSubmitHandler, 1000);
  
})(Drupal, once);

==> dga_rating-2.0.2.zip: dga_rating/js/rating.js <==
(function (Drupal, once) {
  "use strict";

  // DGA Rating Widget Script

  function pctForRating(r) {
    return (Math.max(0, Math.min(5, Number(r) || 0)) / 5) * 100;
  }

  Drupal.behaviors.ratingWidget = {
    attach: function(context, settings) {
      // Find widgets - use direct query if once() fails
      let widgets = [];
      try {
        widgets = once("ratingWidget", "[data-rating-widget]", context);
      } catch (e) {
        // Fallback to direct query
      }

      if (widgets.length === 0) {
        const direct = (context.querySelectorAll || document.querySelectorAll).call(context === document ? document : context, "[data-rating-widget]");
        widgets = Array.from(direct || []);
      }

      if (widgets.length === 0) {
        return;
      }

      widgets.forEach((root) => {
        if (root.hasAttribute('data-rw-initialized')) {
          return;
        }
        root.setAttribute('data-rw-initialized', 'true');

        const rowClosed = root.querySelector("[data-rw-closed]");
        const rowOpen = root.querySelector("[data-rw-opened]");
        const rowSubmitted = root.querySelector("[data-rw-submitted]");
        const btnOpen = root.querySelector("[data-rw-open]");
        const btnClose = root.querySelector("[data-rw-close]");
        const btnSubmit = root.querySelector("[data-rw-submit]");
        const avgText = root.querySelector("[data-rw-average-text]");
        const avgFill = root.querySelector("[data-rw-average-fill]");
        const reviewsEl = root.querySelector("[data-rw-reviews]");
        const starWrap = root.querySelector("[data-rw-stars]");
        const starBtns = root.querySelectorAll("[data-rw-star]");
        const feedbackEl = root.querySelector("[data-rw-feedback]");
        const yourRatingText = root.querySelector("[data-rw-your-rating]");
        const yourFill = root.querySelector("[data-rw-your-fill]");

        let average = Number(root.getAttribute("data-average")) || 0;
        let reviews = root.getAttribute("data-reviews") || "0";
        let url = root.getAttribute("data-url") || window.location.pathname;
        url = url.trim().replace(/\/+$/, '') || '/';
        if (url.match(/^\/[a-z]{2}\//)) {
          url = url.replace(/^\/[a-z]{2}/, '');
          url = url || '/';
        }

        const entityType = root.getAttribute("data-entity-type") || "";
        const entityId = root.getAttribute("data-entity-id") || "";

        const state = { rating: 0, feedback: "", open: false, submitted: false };

        function setClosedView() {
          if (rowClosed) rowClosed.hidden = false;
          if (rowOpen) rowOpen.hidden = true;
          if (rowSubmitted) rowSubmitted.hidden = true;
        }
        function setOpenView() {
          if (rowClosed) rowClosed.hidden = true;
          if (rowOpen) rowOpen.hidden = false;
          if (rowSubmitted) rowSubmitted.hidden = true;
        }
        function setSubmittedView() {
          if (rowClosed) rowClosed.hidden = true;
          if (rowOpen) rowOpen.hidden = true;
          if (rowSubmitted) rowSubmitted.hidden = false;
        }
        function paintAverage() {
          if (avgText) avgText.textContent = average.toFixed(1);
          if (avgFill) avgFill.style.width = pctForRating(average) + "%";
          if (reviewsEl) {
            const reviewsText = reviews === "1" 
              ? (typeof Drupal !== 'undefined' ? Drupal.t('review') : 'review')
              : (typeof Drupal !== 'undefined' ? Drupal.t('reviews') : 'reviews');
            reviewsEl.textContent = reviews + " " + reviewsText;
          }
        }
        function paintYourRating() {
          if (yourRatingText) yourRatingText.textContent = String(state.rating);
          if (yourFill) yourFill.style.width = pctForRating(state.rating) + "%";
          starBtns.forEach((btn) => {
            const v = Number(btn.getAttribute("data-value"));
            btn.setAttribute("aria-checked", v === state.rating ? "true" : "false");
            btn.classList.toggle("is-active", v <= state.rating);
          });
        }

        // Events
        if (btnOpen) {
          btnOpen.addEventListener("click", (e) => {
            e.preventDefault();
            state.open = true;
            setOpenView();
          });
        }

        if (btnClose) {
          btnClose.addEventListener("click", () => {
            state.open = false;
            setClosedView();
          });
        }

        if (feedbackEl) {
          feedbackEl.addEventListener("input", (e) => {
            state.feedback = e.target.value;
          });
        }

        starBtns.forEach((btn) => {
          btn.addEventListener("click", () => {
            state.rating = Number(btn.getAttribute("data-value"));
            paintYourRating();
          });
        });

        // The global handler below will attach to the submit button
        // No need for widget-specific handler since global one works reliably
      });
    },
  };

  // Fallback for delayed content
  setTimeout(() => {
    if (typeof Drupal !== 'undefined' && Drupal.behaviors && Drupal.behaviors.ratingWidget) {
      Drupal.behaviors.ratingWidget.attach(document, Drupal.settings || {});
    }
  }, 1000);

  // GLOBAL HANDLER - Direct button handler that works independently
  function attachGlobalSubmitHandler() {
    const submitButtons = document.querySelectorAll('[data-rw-submit]');

    submitButtons.forEach(function(btn) {
      if (btn.hasAttribute('data-global-handler-attached')) {
        return;
      }

      btn.setAttribute('data-global-handler-attached', 'true');

      btn.onclick = function(e) {
        e.preventDefault();
        e.stopPropagation();
        e.stopImmediatePropagation();

        // Find widget root
        const widget = btn.closest('[data-rating-widget]');
        if (!widget) {
          console.error('DGA Rating: Widget not found');
          alert(typeof Drupal !== 'undefined' ? Drupal.t('ERROR: Widget not found') : 'ERROR: Widget not found');
          return false;
        }

        // Get rating from stars
        let rating = 0;
        const activeStars = widget.querySelectorAll('[data-rw-star].is-active');
        if (activeStars.length > 0) {
          rating = activeStars.length;
        } else {
          const checkedStar = widget.querySelector('[data-rw-star][aria-checked="true"]');
          if (checkedStar) {
            rating = Number(checkedStar.getAttribute('data-value')) || 0;
          }
        }

        if (rating < 1 || rating > 5) {
          alert(typeof Drupal !== 'undefined' ? Drupal.t('Please select a rating before submitting.') : 'Please select a rating before submitting.');
          return false;
        }

        // Get feedback
        const feedbackEl = widget.querySelector('[data-rw-feedback]');
        const feedback = feedbackEl ? feedbackEl.value : '';

        // Get URL
        let url = widget.getAttribute('data-url') || window.location.pathname;
        url = url.trim().replace(/\/+$/, '') || '/';
        if (url.match(/^\/[a-z]{2}\//)) {
          url = url.replace(/^\/[a-z]{2}/, '');
          url = url || '/';
        }

        // Disable button
        const originalText = btn.textContent;
        btn.disabled = true;
        btn.textContent = typeof Drupal !== 'undefined' ? Drupal.t('Submitting...') : 'Submitting...';

        // Prepare payload
        const payload = {
          rating: rating,
          feedback: feedback || '',
          url: url || '/',
          entity_type: 'node',
          entity_id: 0
        };

        // Send request
        const xhr = new XMLHttpRequest();
        xhr.open('POST', '/dga-rating/submit', false);
        xhr.setRequestHeader('Content-Type', 'application/json');

        try {
          xhr.send(JSON.stringify(payload));

          let data;
          try {
            data = JSON.parse(xhr.responseText);
          } catch (e) {
            console.error('DGA Rating: JSON Parse Error:', e);
            const errorMsg = typeof Drupal !== 'undefined' ? Drupal.t('ERROR: Invalid JSON response: @response', {'@response': xhr.responseText.substring(0, 200)}) : 'ERROR: Invalid JSON response: ' + xhr.responseText.substring(0, 200);
            alert(errorMsg);
            btn.disabled = false;
            btn.textContent = originalText;
            return false;
          }

          // Validate response
          if (xhr.status === 200 && data.success && data.rating_id) {

            // Update widget UI
            const rowClosed = widget.querySelector('[data-rw-closed]');
            const rowOpen = widget.querySelector('[data-rw-opened]');
            const rowSubmitted = widget.querySelector('[data-rw-submitted]');

            if (rowClosed) rowClosed.hidden = true;
            if (rowOpen) rowOpen.hidden = true;
            if (rowSubmitted) {
              rowSubmitted.hidden = false;
              const yourRating = rowSubmitted.querySelector('[data-rw-your-rating]');
              if (yourRating)               yourRating.textContent = rating;
            }
          } else {
            console.error('DGA Rating: Submission failed - Status:', xhr.status, 'Success:', data.success, 'Rating ID:', data.rating_id);
            const errorMsg = typeof Drupal !== 'undefined'
              ? Drupal.t('ERROR: Status @status\nMessage: @message', {'@status': xhr.status, '@message': (data.message || Drupal.t('Unknown error'))})
              : 'ERROR: Status ' + xhr.status + '\nMessage: ' + (data.message || 'Unknown error');
            alert(errorMsg);
            btn.disabled = false;
            btn.textContent = originalText;
          }
        } catch (err) {
          console.error('DGA Rating: Exception during submission:', err);
          const errorMsg = typeof Drupal !== 'undefined'
            ? Drupal.t('EXCEPTION: @message\nStatus: @status', {'@message': err.message, '@status': xhr.status})
            : 'EXCEPTION: ' + err.message + '\nStatus: ' + xhr.status;
          alert(errorMsg);
          btn.disabled = false;
          btn.textContent = originalText;
        }

        return false;
      };
    });
  }

  // Attach on DOM ready and once after delay
  if (document.readyState === 'loading') {
    document.addEventListener('DOMContentLoaded', attachGlobalSubmitHandler);
  } else {
    attachGlobalSubmitHandler();
  }

  // Single delayed attach as fallback
  setTimeout(attachGlobalSubmitHandler, 1000);

})(Drupal, once);
